@@ -7,8 +7,10 @@ import json
 import sys
 import os
 
+from rlpython.utils.gc_utils import get_object_by_id
+from rlpython.command_runtime import CommandRuntime
 from rlpython.python_runtime import PythonRuntime
-from rlpython.shell_runtime import ShellRuntime
+from rlpython.completion import Completer
 from rlpython.utils.strings import color
 from rlpython import VERSION_STRING
 
@@ -28,6 +30,7 @@ rlpython {}
 
 DEFAULT_VARIABLES = {
     'pretty_print': True,
+    'repeat_last_command_on_enter': False,
 }
 
 
@@ -43,7 +46,7 @@ class Repl:
                  prompt_ps2=DEFAULT_PROMPT_PS2,
                  history_file=DEFAULT_HISTORY_FILE,
                  history_size=DEFAULT_HISTORY_SIZE,
-                 globals={}, locals={}, **variables):
+                 globals={}, locals={}, commands=[], **variables):
 
         self.banner = banner
         self.warnings = warnings
@@ -52,6 +55,8 @@ class Repl:
         self.ps2 = prompt_ps2
         self.history_file = os.path.expanduser(history_file)
         self.history_size = history_size
+        self.globals = globals
+        self.locals = locals
 
         self.exit_code = 0
 
@@ -60,6 +65,9 @@ class Repl:
             **variables,
         }
 
+        # setup completion
+        self.completer = Completer(repl=self)
+
         # setup readline
         readline.set_auto_history(False)
         readline.parse_and_bind('tab: complete')
@@ -74,17 +82,20 @@ class Repl:
             locals=locals,
         )
 
-        self.shell_runtime = ShellRuntime(
+        self.command_runtime = CommandRuntime(
             repl=self,
         )
 
+        for command in commands:
+            self.install_command(command)
+
         # finish
         self.clear_line_buffer()
         self.setup()
 
     def shutdown(self):
         self.python_runtime.shutdown()
-        self.shell_runtime.shutdown()
+        self.command_runtime.shutdown()
 
         if self.domain == self.DOMAIN.LOCAL:
             self.write_history()
@@ -100,6 +111,14 @@ class Repl:
     def set_domain(self, domain):
         self._domain = domain
 
+    # utils ###################################################################
+    def get_object_by_id(self, object_id):
+        return get_object_by_id(object_id)
+
+    # commands ################################################################
+    def install_command(self, command):
+        self.command_runtime.install_command(command)
+
     # history #################################################################
     def read_history(self):
         self.history = []
@@ -155,13 +174,19 @@ class Repl:
 
         self.write(color(text, fg='red'))
 
+    def write_warning(self, string):
+        self.write(color('WARNING: {}'.format(string), fg='yellow'))
+
+    def write_error(self, string):
+        self.write(color('ERROR: {}'.format(string), fg='red'))
+
     # line buffer #############################################################
     def clear_line_buffer(self):
         self.line_buffer = ''
 
     def validate_line_buffer(self):
         if self.line_buffer.startswith('%'):
-            return self.shell_runtime.validate_source(self.line_buffer)
+            return self.command_runtime.validate_source(self.line_buffer)
 
         return self.python_runtime.validate_source(self.line_buffer)
 
@@ -244,23 +269,19 @@ class Repl:
         self.write_warnings()
 
     def complete(self, text, state):
-        rl_line_buffer = readline.get_line_buffer()
+        line_buffer = readline.get_line_buffer()
 
-        if rl_line_buffer.startswith('%'):
-            rl_line_buffer = rl_line_buffer[1:]
-
-            return self.shell_runtime.complete(
-                text=text,
-                state=state,
-                line_buffer=rl_line_buffer,
-            )
-
-        else:
-            return self.python_runtime.complete(text, state)
+        return self.completer.complete(text, state, line_buffer)
 
     def handle_empty_line(self):
         self.clear_line_buffer()
 
+        if self.variables['repeat_last_command_on_enter']:
+            command = self.history[-1]
+
+            self.write(command + '\n')
+            self.run(command)
+
     def handle_ctrl_d(self):
         self.clear_line_buffer()
         self.write('\n')
@@ -289,7 +310,7 @@ class Repl:
     def run(self, command):
         try:
             if command.startswith('%'):
-                self.exit_code = self.shell_runtime.run(command)
+                self.exit_code = self.command_runtime.run(command)
 
             else:
                 self.exit_code = self.python_runtime.run(command)
